#ifndef dnet_h__
#define dnet_h__

/*
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*  Release date: 01/09/2015
*  Authors: Norbert Bus, Nabil H. Mustafa
*  
*  Description: this module provides a function to compute approximate 
*  hitting sets of small size for disks in the plane.
*
*  If you find this code useful, please cite it by referencing the following paper:
*  @inproceedings{BMR15,
*    author={N. Bus and N. H. Mustafa and S. Ray},
*    title={Geometric Hitting Sets for Disks: Theory and Practice},
*    booktitle={Proceedings of 23rd Annual European Symposium on Algorithms (ESA 2015)},
*    year={2015},
*    pages={}
*  }
*/

#include "CGAL/Exact_predicates_inexact_constructions_kernel.h"
#include "CGAL/Triangulation_vertex_base_with_info_2.h"
#include "CGAL/Triangulation_face_base_with_info_2.h"
#include "CGAL/Delaunay_triangulation_2.h"
#include "CGAL/Circle_2.h"
#include "../enet/enet.h"

namespace dnet{

  ////////////////////////////////// typedefs for CGAL  ///////////////////////////////////////

  struct FaceInfo { 
    std::vector<size_t> diskpoints; 
  };

  typedef CGAL::Exact_predicates_inexact_constructions_kernel  K;

  typedef CGAL::Circle_2<K> Circle;
  typedef CGAL::Vector_2<K> Vector;

  typedef CGAL::Triangulation_face_base_2<K> Tfb;
  typedef CGAL::Triangulation_vertex_base_2<K> Tvb;
  typedef CGAL::Triangulation_data_structure_2<Tvb, Tfb> Tds;
  typedef CGAL::Delaunay_triangulation_2<K, Tds> Delaunay;

  typedef CGAL::Triangulation_face_base_with_info_2<FaceInfo, K> TfbInfo;
  typedef CGAL::Triangulation_vertex_base_with_info_2<size_t, K> TvbInfo;
  typedef CGAL::Triangulation_data_structure_2<TvbInfo, TfbInfo> TdsInfo;
  typedef CGAL::Delaunay_triangulation_2<K, TdsInfo> DelaunayInfo;
  typedef DelaunayInfo::Point Point;
  typedef DelaunayInfo::Edge Edge;
  typedef DelaunayInfo::Face_handle Face_handle;
  typedef DelaunayInfo::Vertex_handle Vertex_handle;
  typedef DelaunayInfo::Face_circulator Face_circulator;


  ////////////////////////////////// dnet functions ///////////////////////////////////////

  extern double bg_c1;  //size of global epsnet is with eps=bg_c1/OPT.
  extern double bg_c2;  //size of phase_epsnet is with eps=bg_c2/OPT.
  extern double bg_c3;  //reweigh a disk only if weight < bg_c3/OPT
  extern double bg_c4;  //reweighing factor
  extern double bg_c5;  //finish algorithm if remaining_hittingset.size < bg_c5*OPT.

  /*!
  * \brief Computes the hitting set with out simplified Bronnimann-Goodrich implementation
  * \param [in,out]  input_points  The input points.
  * \param input_disks             The input disks.
  * \param [in,out]  Hittingset    The hittingset (indices).
  * \param OPT                     The optimal value if known (if zero the algo will guess).
  */
  void bg_solve( std::vector<Point> & input_points, const std::vector<Circle> & input_disks, std::vector<size_t> & Hittingset, size_t OPT = 0);

}
#endif // epsnet_h__
